<?php

namespace ACL\Repositories;

use Doctrine\DBAL\Connection;

class AccessProfile {

    const QUERY_MAX_NRPERFILACESSO = "SELECT MAX(NRPERFILACESSO) as MAXNRPERFILACESSO FROM PERFILACESSO";

    /** @var Connection */
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function getMaxNrPerfilAcesso () {
        return $this->connection->fetchAll(self::QUERY_MAX_NRPERFILACESSO);
    }
    
    public function findAccessControllUtilization($nrorg){
        try {
            $query = "SELECT IDUTCTRLACESWEB
                        FROM PARAMGERAL
                       WHERE NRORG = ".$nrorg;
            return $this->connection->fetchAll($query);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function validateActivation($nrorg){
        $query = "SELECT COUNT(*) AS TOTAL
                    FROM PERFACESSPARC PEPA
                    JOIN PERFILACESSO PERF ON PERF.NRPERFILACESSO = PEPA.NRPERFILACESSO AND PERF.NRORG = PEPA.NRORG
                   WHERE PEPA.IDATIVO = 'S'
                     AND PERF.IDATIVO = 'S'
                     AND PERF.IDPERFILACESSOADM = 'S'
                     AND PERF.NRORG = " . $nrorg;
        try {
            return $this->connection->fetchAll($query);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    public function changeAccessProfile($nrorg, $idutctrlacesweb){
        try {
            $query = "UPDATE PARAMGERAL SET IDUTCTRLACESWEB = '".$idutctrlacesweb."' WHERE NRORG = ".$nrorg;
            $this->connection->executeQuery($query);
            return $idutctrlacesweb;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    public function changeAccessControl($nrorg, $idutlperfilfilial){
        try {
            $query = "UPDATE PARAMGERAL SET IDUTLPERFILFILIAL = '".$idutlperfilfilial."' WHERE NRORG = ".$nrorg;
            $this->connection->executeQuery($query);
            return $idutlperfilfilial;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    public function insertAccessProfile($nrorg, $idutctrlacesweb, $cdpargeral){
        $insert = "INSERT INTO PARAMGERAL
                              (CDPARGERAL, IDUTCTRLACESWEB, NRORG)
                       VALUES (:CDPARGERAL, :IDUTCTRLACESWEB, :NRORG)";

        $params = array(
            "CDPARGERAL"    =>$cdpargeral,
            "IDUTCTRLACESWEB"  => $idutctrlacesweb,
            "NRORG"         => $nrorg
        );

        try
        {
            $std = $this->connection->prepare($insert);
            $std->execute($params);
        }
        catch(\Exception $e)
        {
            throw new \Exception($e->getMessage(), 500);
        };
    }
    
    public function findPerfAcessControl($nrorg, $nrAccessProfile){
        try {
            $nrAccessProfile = ($nrAccessProfile == '' ? null : $nrAccessProfile);
            $query = "SELECT PC.NRPERFILACESSO, PA.NMPERFILACESSO
                        FROM PERFACESSCONTROL PC
                        JOIN PERFILACESSO PA ON
                            PA.NRPERFILACESSO = PC.NRPERFILACESSO
                       WHERE PC.NRORG = ".$nrorg;
            if($nrAccessProfile != null){
                $query .= " AND PC.NRPERFACESSCONT = ".$nrAccessProfile;
            }
            return $this->connection->fetchAll($query);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    } 
    
    public function findPerfAccessByControlProfile($nrorg, $nrAccessProfile){
        try {
            $query = "SELECT NRPERFILACESSO, NMPERFILACESSO, {$nrAccessProfile} AS NRPERFACESSCONT
                        FROM PERFILACESSO 
                       WHERE NRORG = {$nrorg} 
                            AND NRPERFILACESSO NOT IN (SELECT NRPERFILACESSO FROM PERFACESSCONTROL WHERE NRPERFACESSCONT = {$nrAccessProfile})
                            ";
            return $this->connection->fetchAll($query);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }
    
    public function saveAccessProfileByControlProfile($accessProfile)
    {
        $query = "INSERT INTO PERFACESSCONTROL
                              (NRPERFACESSCONT, NRPERFILACESSO,    NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                       VALUES ( ".$accessProfile['NRPERFACESSCONT'].", ".$accessProfile['NRPERFILACESSO'].",   ".$accessProfile['NRORG'].", SYSDATE, ".$accessProfile['NRORG'].", ".$accessProfile['CDOPERINCLUSAO'].")";
                               
        try {
            $std = $this->connection->prepare($query);
            return $std->execute(array());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }
    
    public function deleteControlProfile($nrAccessProfile, $nrControlAccess, $organizationId)
    {
        $query = "DELETE 
                    FROM PERFACESSCONTROL
                   WHERE NRPERFILACESSO   = :NRPERFILACESSO
                     AND NRPERFACESSCONT = :NRPERFACESSCONT
                     AND NRORG      = :NRORG";
        
        $params = array(
            ':NRPERFILACESSO' => $nrAccessProfile,
            ':NRPERFACESSCONT'   => $nrControlAccess,
            ':NRORG'      => $organizationId
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }
    
    public function deleteAssociatedPerfAcessFili($profilesToDelete, $params)
    {
        $queryPerfAcessFili = "DELETE 
                    FROM PERFACESSFILI
                   WHERE NRPERFILACESSO   = :NRPERFILACESSO
                     AND NRORG      = :NRORG";
                     
        $queryPerfAcessControl = "DELETE 
                    FROM PERFACESSCONTROL
                   WHERE NRPERFILACESSO   = :NRPERFILACESSO
                     AND NRORG      = :NRORG";
                     
        
        $params = array(
            ':NRPERFILACESSO' => $profilesToDelete[0],
            ':NRORG'    => $params['NRORG']
        );
        
        try {
            $std = $this->connection->prepare($queryPerfAcessFili);
            $std->execute($params);
            $std = $this->connection->prepare($queryPerfAcessControl);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }
    
    
}