<?php

namespace ACL\Repositories;


use Doctrine\DBAL\Connection;
use Teknisa\Model\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\ORM\EntityManager;
use ACL\Zeedhi\DevOps\Util\Utilities;
use Exception;

class ConfigBot extends RepositoryBase {

    Use TQuery;

    protected $table = 'TELEGRAM_BOT';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $organizationId
     * @return array
     */
    public function findAllConfigBot($conditionsQuery, $groupIdString, $page, $limit, $conditionsSearchQuery): array
    {
        $pageQuery = ($page-1) * $limit;
        $nrorg = getNrorg();
        $query = "SELECT * FROM (SELECT COUNT(1) OVER () TOTAL_LINES_ZHNEXT,TB.NRORG, TB.ID, TB.TOKEN, TB.BOT_ID, TB.NAME, TB.USERNAME, TB.DESCRIPTION,TB.IDATIVO,
        OPERUP.NMOPERADOR AS NMOPERULTATU, OPERIN.NMOPERADOR AS NMOPERINCLUSAO, TB.DTINCLUSAO, TB.DTULTATU,
        rtrim(xmlagg(xmlelement(e,TBG.TELEGRAM_GROUP_ID ,',').extract('//text()') order by TBG.TELEGRAM_GROUP_ID ).GetClobVal(),',') AS GROUP_ID,
        rtrim(xmlagg(xmlelement(e,TG. TITLE ,',').extract('//text()') order by TG. TITLE ).GetClobVal(),',') AS GROUP_NAME,
        rtrim(xmlagg(xmlelement(e,TG.GROUP_ID ,',').extract('//text()') order by TG.GROUP_ID ).GetClobVal(),',') AS ID_GROUP
        FROM TELEGRAM_BOT TB
        LEFT JOIN TELEGRAM_BOT_GROUP TBG ON TB.NRORG = TBG.NRORG AND TB.ID = TBG.TELEGRAM_BOT_ID
        LEFT JOIN TELEGRAM_GROUP TG ON TG.NRORG = TBG.NRORG AND TG.ID = TBG.TELEGRAM_GROUP_ID
        LEFT JOIN OPERADOR OPERIN ON TB.CDOPERINCLUSAO = OPERIN.CDOPERADOR
        LEFT JOIN OPERADOR OPERUP ON TB.CDOPERULTATU = OPERUP.CDOPERADOR
        WHERE TB.NRORG = $nrorg
        $groupIdString
        GROUP BY TB.NRORG,TB.ID, TB.TOKEN, TB.BOT_ID, TB.NAME, TB.USERNAME, TB.DESCRIPTION, TB.IDATIVO, OPERUP.NMOPERADOR, OPERIN.NMOPERADOR,
        TB.DTINCLUSAO, TB.DTULTATU)
        $conditionsSearchQuery
        $conditionsQuery
        OFFSET $pageQuery ROWS FETCH NEXT $limit ROWS ONLY";

        $result =  $this->entityManager->getConnection()->fetchAll($query);
        $total = empty($result) ? 0 : $result[0]['TOTAL_LINES_ZHNEXT'];
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $limit, 'page' => $page, 'total' => $total]];
        foreach ($returnNext["data"] as &$item) {
            $item['GROUP_ID'] = \ACL\Util\AclUtilities::setSelectMultiInQuery($item['GROUP_ID']);
            $item['ID_GROUP'] =  \ACL\Util\AclUtilities::setSelectMultiInQuery($item['ID_GROUP']);
        }
        return $returnNext;
    }

    public function findGroupBotByToken($token, $idgroups): array
    {
        $nrorg = getNrorg();
        $addwhere = '';
        if(count($idgroups) > 0){
            $addwhere =  "AND TG.GROUP_ID NOT IN (:ID_GROUPS)";
        }
        $query = <<<SQL
        SELECT TG.GROUP_ID AS ID_GROUP, TG.TITLE AS NAME
        FROM TELEGRAM_BOT TB
        JOIN TELEGRAM_BOT_GROUP TBG ON TB.NRORG = TBG.NRORG AND TB.ID = TBG.TELEGRAM_BOT_ID
        JOIN TELEGRAM_GROUP TG ON TG.NRORG = TBG.NRORG AND TG.ID = TBG.TELEGRAM_GROUP_ID
        WHERE  TB.NRORG = $nrorg 
        AND TB.TOKEN = :TOKEN
        $addwhere
SQL;

        $params = [
            'TOKEN' => $token,
            'ID_GROUPS' => $idgroups,
        ];
        $types = [
            'ID_GROUPS' => Connection::PARAM_STR_ARRAY,
        ];
        $result =  $this->entityManager->getConnection()->fetchAll($query,$params,$types);
        foreach ($result as &$item) {
            $item['ID_GROUP'] = (int)$item['ID_GROUP'];
       }
        return $result;
    }


    public function findAllGroupBot(): array
    {
        $nrorg = getNrorg();
        $query = "SELECT ID, TITLE 
        FROM TELEGRAM_GROUP
        WHERE  TB.NRORG = $nrorg ";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    public function configBotInsert($token, $id, $botId, $idativo, $name, $username, $description, $nrorg, $cdoperinclusao)
    {
        $query = <<<SQL
            INSERT INTO TELEGRAM_BOT
                        (ID, TOKEN, BOT_ID, NAME, USERNAME, DESCRIPTION, IDATIVO, NRORG, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, 
                        CDOPERINCLUSAO, CDOPERULTATU)
                 VALUES (:ID, :TOKEN, :BOT_ID, :NAME, :USERNAME, :DESCRIPTION, :IDATIVO, :NRORG, SYSDATE, SYSDATE, :NRORGINCLUSAO, 
                        :CDOPERINCLUSAO, :CDOPERINCLUSAO)
SQL;

        $params = array (
            "ID"                => $id,
            "TOKEN"             => $token,
            "BOT_ID"            => $botId,
            "NAME"              => $name,
            "USERNAME"          => $username,
            "DESCRIPTION"       => $description,
            "IDATIVO"           => $idativo,
            "NRORG"             => $nrorg,
            "NRORGINCLUSAO"     => $nrorg,
            "CDOPERINCLUSAO"    => $cdoperinclusao
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function configGroupInsert($id, $groupId, $title, $nrorg, $cdoperinclusao, $idativo)
    {
        $query = <<<SQL
            INSERT INTO TELEGRAM_GROUP
                        (ID, GROUP_ID, TITLE, IDATIVO, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO)
            VALUES (:ID, :GROUP_ID, :TITLE, :IDATIVO, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO)
SQL;

        $params = array (
            "ID"                => $id,
            "GROUP_ID"          => $groupId,
            "TITLE"             => $title,
            "IDATIVO"           => $idativo,
            "NRORG"             => $nrorg,
            "NRORGINCLUSAO"     => $nrorg,
            "CDOPERINCLUSAO"    => $cdoperinclusao
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function configBotGroupInsert($idBotGroup, $id, $idGroup, $nrorg, $cdoperinclusao, $idativo)
    {
        $query = <<<SQL
            INSERT INTO TELEGRAM_BOT_GROUP
                        (ID, TELEGRAM_BOT_ID, TELEGRAM_GROUP_ID, IDATIVO, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO)
            VALUES (:ID, :TELEGRAM_BOT_ID, :TELEGRAM_GROUP_ID, :IDATIVO, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO)
SQL;

        $params = array (
            "ID"                => $idBotGroup,
            "TELEGRAM_BOT_ID"   => $id,
            "TELEGRAM_GROUP_ID" => $idGroup,
            "IDATIVO"           => $idativo,
            "NRORG"             => $nrorg,
            "NRORGINCLUSAO"     => $nrorg,
            "CDOPERINCLUSAO"    => $cdoperinclusao
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function configBotUpdate($description, $id, $idativo, $cdoperadorultatu)
    {
        $nrorg = getNrorg();
        $query = "UPDATE TELEGRAM_BOT
                    SET DESCRIPTION = :DESCRIPTION,
                    IDATIVO = :IDATIVO,
                    CDOPERULTATU = :CDOPERULTATU,
                    DTULTATU    = SYSDATE
                    WHERE ID = :ID
                    AND NRORG = $nrorg ";

        $params = array ('DESCRIPTION' => $description, 'ID' => $id, 'IDATIVO' => $idativo, 'CDOPERULTATU'  => $cdoperadorultatu);

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function findBotAntigo($id)
    {

        $query = "SELECT *FROM TELEGRAM_BOT
                  WHERE ID = :ID
                  AND NRORG  = :NRORG";

        $params = array (
            "ID" => $id,
            "NRORG"         => InstanceProvider::getEnvironment()->getCurrentOrganizationId(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function deleteBot($params)
    {
        $nrorg = getNrorg();
        $query = "DELETE FROM TELEGRAM_BOT WHERE ID = :ID AND NRORG = $nrorg";
        InstanceProvider::getEntityManager()->getConnection()->executeStatement($query, $params);
    }

    public function deleteBotGroup($params)
    {
        $nrorg = getNrorg();
        $query = "DELETE FROM TELEGRAM_BOT_GROUP WHERE TELEGRAM_BOT_ID = :TELEGRAM_BOT_ID AND NRORG = $nrorg";
        InstanceProvider::getEntityManager()->getConnection()->executeStatement($query, $params);
    }

    public function deleteGroup($params)
    {
        $nrorg = getNrorg();
        $query = "DELETE FROM TELEGRAM_GROUP WHERE ID = :ID AND NRORG = $nrorg";
        InstanceProvider::getEntityManager()->getConnection()->executeStatement($query, $params);
    }

    public function getIdGroup($params)
    {
        $nrorg = getNrorg();
        $query = "SELECT ID
        FROM TELEGRAM_GROUP
        WHERE GROUP_ID = :GROUP_ID  AND NRORG = $nrorg";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }
}
