<?php

namespace ACL\Repositories;

use ACL\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\DBAL\Exception;
use Doctrine\ORM\EntityManager;

class Filioper extends RepositoryBase {

    Use TQuery;

    protected $table = 'FILIOPER';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $nrorg
     * @param string $filters
     * @return array[]
     */
    public function findFilioper($nrorg, $filters = ''): array
    {
        $query = <<<SQL
            SELECT DISTINCT CDOPERADOR, CDFILIAL, IDEXIBAVISO, NRORGFILIAL, DTULTATUFOPE, NRORG
                   DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, 
                   CDOPERULTATU, IDATIVO, NMFILIAL, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE, NRINSJURFILI,
                   NMPAIS, NMESTADO, NMMUNICIPIO, NMREGIAO
              FROM (SELECT CDOPERADOR, FIOP.CDFILIAL, IDEXIBAVISO, NRORGFILIAL, DTULTATUFOPE,
                           FIOP.NRORG, FIOP.DTINCLUSAO, FIOP.DTULTATU, FIOP.NRORGINCLUSAO,
                           FIOP.CDOPERINCLUSAO, FIOP.NRORGULTATU, FIOP.CDOPERULTATU, FIOP.IDATIVO,
                           FILI.NMFILIAL, FIOP.DTVALIDADE, FILI.NRINSJURFILI,
                            PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO
                      FROM FILIOPER FIOP
                      JOIN FILIAL FILI ON FILI.NRORG = FIOP.NRORG
                                      AND FILI.CDFILIAL = FIOP.CDFILIAL
                      JOIN ENDEFILI EM 
                            ON FILI.CDFILIAL = EM.CDFILIAL
                            AND FILI.NRORG = EM.NRORG
                            AND EM.IDTPENDEFILI = 'P'
                    LEFT JOIN PAIS PA 
                            ON EM.CDPAIS = PA.CDPAIS
                            AND EM.NRORG = PA.NRORG
                    LEFT JOIN ESTADO ES
                            ON EM.SGESTADO = ES.SGESTADO
                            AND EM.CDPAIS = ES.CDPAIS
                            AND EM.NRORG = ES.NRORG
                    LEFT JOIN MUNICIPIO MU 
                            ON EM.CDMUNICIPIO = MU.CDMUNICIPIO
                           AND EM.SGESTADO = MU.SGESTADO
                           AND EM.CDPAIS = MU.CDPAIS
                           AND EM.NRORG = MU.NRORG
                    LEFT JOIN REGIAO RE 
                            ON FILI.CDREGIAO = RE.CDREGIAO
                            AND EM.SGESTADO = RE.SGESTADO
                            AND EM.CDPAIS = RE.CDPAIS
                            AND EM.NRORG = RE.NRORG)
                                      
              WHERE 1 = 1
              AND NRORG = :NRORG
              $filters
SQL;
        $params = array(
            'NRORG' => $nrorg
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    /**
     * @param $cdoperador
     * @param $cdfilial
     * @param $nrorg
     * @param $cdoperinclusao
     * @param null $dtvalidade
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Exception
     */
    public function create($cdoperador, $cdfilial, $nrorg, $cdoperinclusao, $dtvalidade = null)
    {
        $query = "INSERT INTO FILIOPER
                              (CDOPERADOR, CDFILIAL, IDEXIBAVISO, DTULTATUFOPE, NRORG, DTINCLUSAO,
                              DTULTATU, NRORGINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORGFILIAL,
                              IDATIVO, DTVALIDADE)
                      VALUES (:CDOPERADOR, :CDFILIAL, 'S', SYSDATE, :NRORG, SYSDATE,
                              SYSDATE, :NRORG, :NRORG, :CDOPERINCLUSAO, :NRORG, 'S', :DTVALIDADE)";

        $params = array (
            "CDOPERADOR" => $cdoperador,
            "NRORG" => $nrorg,
            "CDOPERINCLUSAO" => $cdoperinclusao,
            "CDFILIAL" => $cdfilial,
            "DTVALIDADE" => $dtvalidade
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    /**
     * @param $cdoperador
     * @param $cdfilial
     * @param $organizationId
     * @param $dtValidade
     * @param $idativo
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function updateFilioper($cdoperador, $cdfilial, $organizationId, $dtValidade, $idativo)
    {
        $query = "UPDATE FILIOPER
                     SET DTVALIDADE      = :DTVALIDADE,
                         IDATIVO         = :IDATIVO
                   WHERE CDFILIAL        = :CDFILIAL
                     AND CDOPERADOR      = :CDOPERADOR
                     AND NRORG           = :NRORG";

        $params = array(
            "CDFILIAL"          => $cdfilial,
            "CDOPERADOR"        => $cdoperador,
            "NRORG"             => $organizationId,
            "DTVALIDADE"        => $dtValidade,
            "IDATIVO"           => $idativo
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    /**
     * @param $cdOperador
     * @param $cdFilial
     * @param $nrOrg
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Exception
     */
    public function deleteFilioper($cdOperador, $cdFilial, $nrOrg)
    {
        $query = "DELETE
                    FROM FILIOPER
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND CDFILIAL   = :CDFILIAL
                     AND NRORG      = :NRORG";

        $params = array(
            "CDOPERADOR" => $cdOperador,
            "CDFILIAL"   => $cdFilial,
            "NRORG"      => $nrOrg
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function findFiliOperAntigo($cdoperador, $cdfilial)
    {

        $query = "SELECT DTVALIDADE FROM FILIOPER
                WHERE CDOPERADOR = :CDOPERADOR
                AND CDFILIAL = :CDFILIAL
                AND NRORG  = :NRORG";

        $params = array (
            "CDFILIAL" => $cdfilial,
            "CDOPERADOR" => $cdoperador,
            "NRORG"         => InstanceProvider::getEnvironment()->getCurrentOrganizationId(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }
}
