<?php

namespace ACL\Repositories;

use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\DBAL\Driver\Exception;
use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class Opergrupop extends RepositoryBase {
    use TQuery;

    protected $table = 'OPERGRUPOP';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $cdoperador
     * @param $nrorg
     * @return array
     */
    public function findOpergrupopByOperador($cdoperador, $nrorg): array
    {
        $query = <<<SQL
            SELECT O.NRORG, O.CDOPERADOR, O.CDGRUPOPER, G.DSGRUPOPER, O.IDMENUPRINOP
              FROM OPERGRUPOP O, GRUPOPER G
             WHERE O.CDGRUPOPER = G.CDGRUPOPER 
               AND O.CDOPERADOR = :CDOPERADOR
               AND O.NRORG = :NRORG
SQL;
        $params = array(
            'CDOPERADOR' => $cdoperador,
            'NRORG' => $nrorg
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function findGrupoperNotInOpergrupop($cdoperador, $nrorg): array
    {
        $query = <<<SQL
            SELECT G.CDGRUPOPER, G.DSGRUPOPER, G.NRORG
              FROM GRUPOPER G
             WHERE G.CDGRUPOPER NOT IN (SELECT CDGRUPOPER 
                                          FROM OPERGRUPOP
                                         WHERE CDOPERADOR =:CDOPERADOR)
               AND G.IDATIVO = 'S'
               AND G.NRORG = :NRORG
          ORDER BY G.CDGRUPOPER, G.DSGRUPOPER, G.NRORG
SQL;
        $params = array(
            'CDOPERADOR' => $cdoperador,
            'NRORG' => $nrorg
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    /**
     * @param $cdgrupoper
     * @param $cdoperador
     * @param $idmenuprinop
     * @param $nrorg
     * @param $cdoperinclusao
     * @throws Exception
     * @throws \Exception
     */
    public function createOpergrupop($cdgrupoper, $cdoperador, $idmenuprinop, $nrorg, $cdoperinclusao)
    {
        $query = "INSERT INTO OPERGRUPOP
                              (CDGRUPOPER, CDOPERADOR, DTULTATUOPGR, IDMENUPRINOP, NRORG, DTINCLUSAO,
                               NRORGINCLUSAO, CDOPERINCLUSAO, IDATIVO)
                      VALUES (:CDGRUPOPER, :CDOPERADOR, SYSDATE, :IDMENUPRINOP, :NRORG, SYSDATE,
                              :NRORG, :CDOPERINCLUSAO, 'S')";

        $params = array (
            "CDGRUPOPER" => $cdgrupoper,
            "CDOPERADOR" => $cdoperador,
            "IDMENUPRINOP" => $idmenuprinop,
            "NRORG" => $nrorg,
            "CDOPERINCLUSAO" => $cdoperinclusao,
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    /**
     * @param $cdgrupoper
     * @param $cdoperador
     * @param $idmenuprinop
     * @throws Exception
     * @throws \Exception
     */
    public function updateOpergrupop($cdgrupoper, $cdoperador, $idmenuprinop, $nrorg)
    {
        $query = "
               UPDATE OPERGRUPOP
                  SET IDMENUPRINOP = :IDMENUPRINOP,
                  DTULTATU         = SYSDATE,
                  CDOPERULTATU     = :CDOPERULTATU
                WHERE CDGRUPOPER = :CDGRUPOPER 
                  AND CDOPERADOR = :CDOPERADOR
                  AND NRORG = :NRORG";

        $params = array(
            "IDMENUPRINOP" => $idmenuprinop,
            "CDGRUPOPER" => $cdgrupoper,
            "NRORG" => $nrorg,
            "CDOPERULTATU" => InstanceProvider::getLibEnvironment()->getCurrentUserId(),
            "CDOPERADOR" => $cdoperador
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    /**
     * @param $cdgrupoper
     * @param $cdoperador
     * @param $nrorg
     * @throws Exception
     * @throws \Exception
     */
    public function daleteOpergrupop($cdgrupoper, $cdoperador, $nrorg)
    {
        $query = "DELETE
                    FROM OPERGRUPOP
                   WHERE CDGRUPOPER = :CDGRUPOPER
                     AND CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";

        $params = array(
            "CDGRUPOPER" => $cdgrupoper,
            "CDOPERADOR" => $cdoperador,
            "NRORG"      => $nrorg
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }
}
