<?php

namespace ACL\Repositories;

use ACL\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\DBAL\Driver\Exception;
use Doctrine\ORM\EntityManager;

class Paramgeral extends RepositoryBase {

    Use TQuery;

    protected $table = 'PARAMGERAL';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $organizationId
     * @return array
     */
    public function findParamgeral($organizationId): array
    {
        $query = <<<SQL
            SELECT IDGERACDOPERADOR, IDCPFREQUIRED
              FROM PARAMGERAL
             WHERE NRORG = :NRORG
SQL;
        $params = array(
            "NRORG" => $organizationId
        );
        $finalQuery = $this->getFinalQuery($query);
        $data = $this->entityManager->getConnection()->fetchAssociative($finalQuery, $params);
        return $this->buildResponseData($data);
    }

    /**
     * @param $idutctrlacesweb
     * @param $cdpargeral
     * @param $nrorg
     * @throws Exception
     * @throws \Exception
     */
    public function createParamgeralByAccessControl($idutctrlacesweb, $cdpargeral, $nrorg)
    {
        $insert = "INSERT INTO PARAMGERAL
                              (CDPARGERAL, IDUTCTRLACESWEB, NRORG)
                       VALUES (:CDPARGERAL, :IDUTCTRLACESWEB, :NRORG)";

        $params = array(
            "IDUTCTRLACESWEB" => $idutctrlacesweb,
            "CDPARGERAL" => $cdpargeral,
            "NRORG" => $nrorg
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($insert);
            $std->executeQuery($params);
        }
        catch(\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function updateParamgeralByAccessControl($idutctrlacesweb, $nrorg){
        try {
            $query = "UPDATE PARAMGERAL 
                         SET IDUTCTRLACESWEB = :IDUTCTRLACESWEB
                       WHERE NRORG = :NRORG";

            $params = array(
                "IDUTCTRLACESWEB" => $idutctrlacesweb,
                "NRORG" => $nrorg
            );

            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }

    public function findAccessProfileOld($nrorg)
    {

        $query = "SELECT *FROM PARAMGERAL
                  WHERE NRORG = :NRORG";

        $params = array (
            "NRORG" => $nrorg
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }

}
