<?php

namespace ACL\Repositories;

use ACL\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\ORM\EntityManager;

class Parceria extends RepositoryBase {

    Use TQuery;

    protected $table = 'PARCERIA';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    public function createParceria($nrparcnegocio, $organizationId, $cdoperadorInclusao, $cdtipoparceiro, $nrparceria)
    {
        $query = "INSERT INTO PARCERIA
                               (NRPARCERIA, NRPARCNEGOCIO, CDTIPOPARCEIRO, NRORG, IDATIVO,
                               DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                       VALUES (:NRPARCERIA, :NRPARCNEGOCIO, :CDTIPOPARCEIRO, :NRORG, :IDATIVO,
                               SYSDATE, :NRORG, :CDOPERINCLUSAO)";

        $params = array(
            "NRPARCERIA"     => $nrparceria,
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "NRORG"          => $organizationId,
            "CDTIPOPARCEIRO" => $cdtipoparceiro,
            "IDATIVO"        => 'S',
            "CDOPERINCLUSAO" => $cdoperadorInclusao
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateIdativoInParceria($userId, $organizationId, $businessPartnership, $activeId)
    {
        $query = "UPDATE PARCERIA
                      SET IDATIVO        = :IDATIVO,
                          CDOPERULTATU   = :CDOPERULTATU,
                          NRORGULTATU    = :NRORGULTATU,
                          DTULTATU       = SYSDATE
                    WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                      AND NRORG          = :NRORG
                      AND CDTIPOPARCEIRO = :CDTIPOPARCEIRO";

        $params = array(
            "NRORG"          => $organizationId,
            "NRPARCNEGOCIO"  => $businessPartnership,
            "NRORGULTATU"    => $organizationId,
            "CDOPERULTATU"   => $userId,
            "IDATIVO"        => $activeId,
            "CDTIPOPARCEIRO" => "INTERLOCUTOR"
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function hasParceriaTypeInterlocutor($organizationId, $businessPartnership)
    {
        $query = "SELECT COUNT(*) AS COUNT
                     FROM PARCERIA
                    WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                      AND NRORG          = :NRORG
                      AND CDTIPOPARCEIRO = :CDTIPOPARCEIRO";

        $params = array(
            "NRORG"          => $organizationId,
            "NRPARCNEGOCIO"  => $businessPartnership,
            "CDTIPOPARCEIRO" => "INTERLOCUTOR"
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result[0]["COUNT"];
        }catch(\Exception $e){
            throw new \Exception($e->getMessage(), 500);
        };
    }
}
