<?php

namespace ACL\Repositories;


use ACL\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\ORM\EntityManager;

class Parcnegocio extends RepositoryBase {

    Use TQuery;

    protected $table = 'PARCNEGOCIO';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $organizationId
     * @return array
     */
    public function findAllParcnegocio($organizationId): array
    {
        $query = <<<SQL
            SELECT NRPARCNEGOCIO, CDSEGMER, CDTIPOPARCPRINCIPAL, CDTIPOINSCRICAO,
                   IDIDIOMA, NMPRINCIPALPARC, NMSECUNDARIPARC, DTNASCIFUNDPARC,
                   NRINSCRICAOPARC, IDPESSOAFISICA, IDINSTITUICAO, DSDEPARTAMENTO
              FROM PARCNEGOCIO
             WHERE NRORG = :NRORG
               AND IDATIVO = 'S'
SQL;
        $params = array(
            "NRORG" => $organizationId
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function createParcnegocio($nrparcnegocio, $organizationId, $nmoperador, $nrinscricaoparc, $cdtipoinscricao,
                                      $cdoperinclusao, $idativo, $cdtipoparcprincipal, $idpessoafisica, $idinstituicao,
                                      $idparcfundido)
    {
        $query = "INSERT INTO PARCNEGOCIO
                              (NRPARCNEGOCIO, NRORG, NMPRINCIPALPARC, NMSECUNDARIPARC, NRINSCRICAOPARC, IDATIVO,
                              DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, CDTIPOPARCPRINCIPAL, CDTIPOINSCRICAO,
                              IDPESSOAFISICA, IDINSTITUICAO, IDPARCFUNDIDO)
                      VALUES (:NRPARCNEGOCIO, :NRORG, :NMOPERADOR, :NMOPERADOR, :NRINSCRICAOPARC, :IDATIVO,
                                SYSDATE, :NRORG, :CDOPERINCLUSAO, :CDTIPOPARCPRINCIPAL, :CDTIPOINSCRICAO,
                              :IDPESSOAFISICA, :IDINSTITUICAO, :IDPARCFUNDIDO)";

        $params = array (
            "NRPARCNEGOCIO"       => $nrparcnegocio,
            "NRORG"               => $organizationId,
            "NMOPERADOR"          => $nmoperador,
            "NRINSCRICAOPARC"     => $nrinscricaoparc,
            "IDATIVO"             => $idativo,
            "CDOPERINCLUSAO"      => $cdoperinclusao,
            "CDTIPOINSCRICAO"     => $cdtipoinscricao,
            "CDTIPOPARCPRINCIPAL" => $cdtipoparcprincipal,
            "IDPESSOAFISICA"      => $idpessoafisica,
            "IDINSTITUICAO"       => $idinstituicao,
            "IDPARCFUNDIDO"       => $idparcfundido
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function createParceria($nrparcnegocio, $organizationId, $cdoperadorInclusao, $cdtipoparceiro, $nrparceria)
    {
        $query = "INSERT INTO PARCERIA
                               (NRPARCERIA, NRPARCNEGOCIO, CDTIPOPARCEIRO, NRORG, IDATIVO,
                               DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                       VALUES (:NRPARCERIA, :NRPARCNEGOCIO, :CDTIPOPARCEIRO, :NRORG, :IDATIVO,
                               SYSDATE, :NRORG, :CDOPERINCLUSAO)";

        $params = array(
            "NRPARCERIA"     => $nrparceria,
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "NRORG"          => $organizationId,
            "CDTIPOPARCEIRO" => $cdtipoparceiro,
            "IDATIVO"        => 'S',
            "CDOPERINCLUSAO" => $cdoperadorInclusao
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateParcnegocio($nrinscricaoparc, $cdtipoinscricao, $nmoperador, $nrorg, $cdoperultatu, $nrparcnegocio)
    {
        $query = "UPDATE PARCNEGOCIO
                      SET NRINSCRICAOPARC = :NRINSCRICAOPARC,
                          CDTIPOINSCRICAO = :CDTIPOINSCRICAO,
                          NMPRINCIPALPARC = :NMOPERADOR,
                          NMSECUNDARIPARC = :NMOPERADOR,
                          DTULTATU        =  SYSDATE,
                          NRORGULTATU     = :NRORG,
                          CDOPERULTATU    = :CDOPERULTATU
                    WHERE NRPARCNEGOCIO   = :NRPARCNEGOCIO
                      AND NRORG           = :NRORG";

        $params = array(
            "NRPARCNEGOCIO"   => $nrparcnegocio,
            "NRORG"           => $nrorg,
            "NMOPERADOR"      => $nmoperador,
            "NRINSCRICAOPARC" => $nrinscricaoparc,
            "CDOPERULTATU"    => $cdoperultatu,
            "CDTIPOINSCRICAO" => $cdtipoinscricao,
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage(), 500);
        };
    }
}
