<?php

namespace ACL\Repositories;

use ACL\Traits\TQuery;
use Doctrine\DBAL\Driver\Exception;
use Doctrine\ORM\EntityManager;
use ACL\Helpers\Instances\InstanceProvider;

class Perfacessparc extends RepositoryBase {

    Use TQuery;

    protected $table = 'PERFACCESSPARC';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $nrperfacessparc
     * @param $nrorg
     * @param $nrparcnegocio
     * @param $nrperfilacesso
     * @param $cdoperinclusao
     * @param $dtvalidade
     * @throws Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function savePerfaccessparc($nrperfacessparc, $nrorg, $nrparcnegocio, $nrperfilacesso,
                                       $cdoperinclusao, $dtvalidade)
    {
        $query = "INSERT INTO PERFACESSPARC
                              (NRPERFACESSPARC, NRORG, NRPARCNEGOCIO, NRPERFILACESSO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTVALIDADE)
                       VALUES (:NRPERFACESSPARC, :NRORG, :NRPARCNEGOCIO, :NRPERFILACESSO, SYSDATE, :NRORG, :CDOPERINCLUSAO, :DTVALIDADE)";

        $params = array(
            "NRPERFACESSPARC"   => $nrperfacessparc,
            "NRORG"             => $nrorg,
            "NRPARCNEGOCIO"     => $nrparcnegocio,
            "NRPERFILACESSO"    => $nrperfilacesso,
            "CDOPERINCLUSAO"    => $cdoperinclusao,
            "DTVALIDADE"        => $dtvalidade,
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrperfacessparc
     * @param $dtValidade
     * @param $nrorg
     * @throws Exception
     * @throws \Exception
     */
    public function updatePerfaccessparc($nrperfacessparc, $dtValidade, $nrorg)
    {
        $query = "UPDATE PERFACESSPARC
                     SET DTVALIDADE = :DTVALIDADE
                   WHERE NRPERFACESSPARC = :NRPERFACESSPARC
                     AND NRORG           = :NRORG";

        $params = array(
            "NRPERFACESSPARC"   => $nrperfacessparc,
            "DTVALIDADE"        => $dtValidade,
            "NRORG"             => $nrorg
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function deletePerfaccessparc($nrperfacessparc, $organizationId)
    {
        $query = "DELETE
                    FROM PERFACESSPARC
                   WHERE NRPERFACESSPARC = :NRPERFACESSPARC
                     AND NRORG           = :NRORG";

        $params = array(
            'NRPERFACESSPARC'   => $nrperfacessparc,
            'NRORG'             => $organizationId
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        };
    }

    public function findValAntigoPerfAcess($partnerAccessProfileNumber)
    {

        $query = "SELECT *FROM PERFACESSPARC
                  WHERE NRPERFACESSPARC = :NRPERFACESSPARC
                  AND NRORG  = :NRORG";

        $params = array (
            "NRPERFACESSPARC" => $partnerAccessProfileNumber,
            "NRORG"         => InstanceProvider::getEnvironment()->getCurrentOrganizationId(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function findOperadorPerfilAcess($nrParcNegocio)
    {

        $query = "SELECT CDOPERADOR, NMOPERADOR FROM OPERADOR
                  WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO AND NRORG = :NRORG";

        $params = array (
            "NRPARCNEGOCIO" => $nrParcNegocio,
            "NRORG"         => InstanceProvider::getEnvironment()->getCurrentOrganizationId(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function findPerfilAcess($nrperfilacesso)
    {

        $query = "SELECT NMPERFILACESSO FROM PERFILACESSO
                    WHERE NRPERFILACESSO = :NRPERFILACESSO AND NRORG = :NRORG";
        $params = array (
            "NRPERFILACESSO" => $nrperfilacesso,
            "NRORG"         => InstanceProvider::getEnvironment()->getCurrentOrganizationId(),
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }
}
