<?php

namespace ACL\Repositories;

use ACL\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\DBAL\Exception;
use Doctrine\ORM\EntityManager;

class PerfilacessoLabel extends RepositoryBase {

    Use TQuery;

    protected $table = 'PERFILACESSO_LABEL';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $nrperfilacesso
     * @param $organizationId
     * @return array
     * @throws Exception
     */
    public function findPerfilacessoLabelByNrperfilacesso($nrperfilacesso, $nrorg): array
    {
        $query = <<<SQL
            SELECT ID, LABEL, NRPERFILACESSO
              FROM PERFILACESSO_LABEL
             WHERE NRPERFILACESSO = :NRPERFILACESSO
               AND NRORG = :NRORG
SQL;
        $params = array(
            'NRPERFILACESSO' => $nrperfilacesso,
            "NRORG" => $nrorg
        );
        return $this->entityManager->getConnection()->fetchAllAssociative($query, $params);
    }

    /**
     * @param $nrperfilacesso
     * @param $labelsQueryIn
     * @param $nrorg
     * @return array
     * @throws Exception
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function findPerfilacessoLabelByNrperfilacessoAndLabels($nrperfilacesso, $labelsQueryIn, $nrorg): array
    {
        $query = <<<SQL
            SELECT ID, LABEL, NRPERFILACESSO
              FROM PERFILACESSO_LABEL
             WHERE NRPERFILACESSO = :NRPERFILACESSO
               AND NRORG = :NRORG
               $labelsQueryIn
SQL;
        $params = array(
            'NRPERFILACESSO' => $nrperfilacesso,
            "NRORG" => $nrorg
        );
        try {
            return $this->entityManager->getConnection()->fetchAllAssociative($query, $params);
        } catch (\Doctrine\DBAL\Driver\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }

    /**
     * @param $id
     * @param $label
     * @param $nrperfilacesso
     * @param $nrorg
     * @param $idativo
     * @param $cdoperinclusao
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws Exception
     */
    public function savePerfilacessoLabel($id, $label, $nrperfilacesso, $nrorg, $idativo, $cdoperinclusao)
    {
        $query = "INSERT INTO PERFILACESSO_LABEL
                              (ID, LABEL, NRPERFILACESSO, NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                       VALUES (:ID, :LABEL, :NRPERFILACESSO, :NRORG, :IDATIVO, SYSDATE, :NRORG, :CDOPERINCLUSAO)";

        $params = array(
            'ID' => $id,
            'LABEL' => $label,
            'NRPERFILACESSO' => $nrperfilacesso,
            'NRORG' => $nrorg,
            "IDATIVO" => $idativo,
            "CDOPERINCLUSAO" => $cdoperinclusao,
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Doctrine\DBAL\Driver\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }

    /**
     * @param $nrperfilacesso
     * @param $labelsQueryIn
     * @param $nrorg
     * @throws Exception
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function deletePerfilacessoLabel($nrperfilacesso, $labelsQueryIn, $nrorg)
    {
        $query = "
            DELETE 
              FROM PERFILACESSO_LABEL
             WHERE NRPERFILACESSO = :NRPERFILACESSO
               AND NRORG = :NRORG
               $labelsQueryIn";

        $params = array(
            'NRPERFILACESSO' => $nrperfilacesso,
            "NRORG" => $nrorg
        );

        try {
            $std = $this->entityManager->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Doctrine\DBAL\Driver\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }
}
