<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Traits\TCrud;
use Exception;
use ACL\Util\AclUtilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class PerfilacessoLabel
{

    use TCrud;

    /**
     * @var int
     */
    public $majorVersionInFile = 0;

    /**
     * @var int
     */
    public $oldMajorVersion = 0;

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;
    /**
     * @var \ACL\Repositories\PerfilacessoLabel
     */
    private $perfilacessoLabelRepository;

    /**
     * @var array
     */
    private $nodes = [];

    public function __construct()
    {
        $this->libEnvironment = InstanceProvider::getLibEnvironment();
        $this->perfilacessoLabelRepository = InstanceProvider::getRepository()->getPerfilacessoLabel();
    }

    /**
     * @param $nrperfilacesso
     * @param $oldMajorVersion
     * @param bool $versionSave
     * @param $isNext
     * @param $productId
     * @param $nrpperfilacesso
     * @return mixed
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
   public function findPerfilacessoLabel($nrperfilacesso, $oldMajorVersion, bool $versionSave, $isNext, $productId, $nrpperfilacesso, $labelTela)
   {
       if (!$isNext) {
           $this->handlePerfilacessoLabelByOldAcl($productId, $nrpperfilacesso, $labelTela);
       }


       $perfilacessoLabels = $this->perfilacessoLabelRepository->findPerfilacessoLabelByNrperfilacesso($nrperfilacesso, getNrorg());
       $this->nodes = $this->getFileAclLabelsJson();

       $this->oldMajorVersion = $oldMajorVersion;

       if ($versionSave) {
           $this->setMajorVersionInFile($this->nodes);
           InstanceProvider::getService()->getOperadorLabelVersao()->save($this->majorVersionInFile);
       }

       return $this->setLabelsVisibility($perfilacessoLabels, $this->nodes);
   }

    /**
     * @return mixed
     * @throws Exception
     */
   private function getFileAclLabelsJson()
   {
       $productBasePath = InstanceProvider::getLibUtilities()->getProductBasePath();
       $aclLabelsJson = json_decode(@file_get_contents($productBasePath . 'aclLabels.json'), true);

       if (!$aclLabelsJson) {
           throw new Exception('Arquivo "aclLabels.json" não encontrado', 500);
       }

       return $aclLabelsJson;
   }

    /**
     * @param $perfilacessoLabels
     * @param $labelsFile
     * @return mixed
     * @throws Exception
     */
   private function setLabelsVisibility($perfilacessoLabels, &$labelsFile)
   {
       foreach ($labelsFile as &$label) {
           if (!empty($label['data']['version']) && $label['data']['version'] > $this->oldMajorVersion) {
               $label['data']['new'] = true;
               if (!empty($label['data']['parent'])) {
                   $this->setParentNodeIsNew($this->nodes, $label['data']['parent']);
               }
           }

           foreach ($perfilacessoLabels as $row) {
               if ($label['data']['type'] !== 'GROUPED' && $row['LABEL'] == $label['data']['label']) {
                   $label['data']['checked'] = true;
               }
           }

           if ($this->hasChildren($label)) {
               $this->setLabelsVisibility($perfilacessoLabels, $label['children']);
           }
       }
       return $labelsFile;
   }

   private function setParentNodeIsNew(&$labelsFile, $parent)
   {
       foreach ($labelsFile as &$label) {
           if ($label['data']['type'] !== 'GROUPED' && $label['data']['label'] === $parent) {
               $label['data']['new'] = true;
               if(!empty($label['data']['parent'])) {
                   $this->setParentNodeIsNew($this->nodes, $label['data']['parent']);
               }
           } else if ($label['data']['type'] === 'GROUPED' && $label['title'] === $parent){
               $label['data']['new'] = true;
           }

           if($this->hasChildren($label)) {
               $this->setParentNodeIsNew($label['children'], $parent);
           }
       }
   }

    /**
     * @param $node
     * @return bool
     */
   private function hasChildren($node): bool
   {
       return \Zeedhi\Framework\Util\Functions::arrayKeyExists('children', $node) && is_array($node['children']) && count($node['children']);
   }

    /**
     * @return int
     * @throws Exception
     */
    private function getMajorVersionInBD(): int
    {
        $data = InstanceProvider::getService()->getOperadorLabelVersao()->findByOperadorAndProject();
        if (empty($data)) {
            return 1;
        }
        return intval($data['NRULTVERSAO']);
    }

    /**
     * @param $labelsFile
     */
    public function setMajorVersionInFile($labelsFile)
    {
        foreach ($labelsFile as $label) {
            if ($label['data']['version'] > $this->majorVersionInFile) {
                $this->majorVersionInFile = $label['data']['version'];
            }
            $this->setMajorVersionInFile($label['children']);
        }
    }

    /**
     * @param $projectId
     * @param $nrperfilacesso
     * @throws \Doctrine\DBAL\Exception
     * @throws Exception
     */
    private function handlePerfilacessoLabelByOldAcl($projectId, $nrperfilacesso, $labelTela)
    {
        $elements = InstanceProvider::getRepository()->getZhuAcl()->getElementsByNrperfilacesso($projectId, $nrperfilacesso, getNrorg());

        foreach ($elements as $element) {
            if($element['TYPE'] == 'M') {
                $this->createPerfilacessoLabelByOldAcl($element['UNIQUE_ID'], $nrperfilacesso, $labelTela);
            } else {
                $menuId = $this->findMenuIdByContainerIdInAclLabels($this->getFileAclLabelsJson(), $element['UNIQUE_ID']);
                if ($menuId) {
                    $this->createPerfilacessoLabelByOldAcl($menuId, $nrperfilacesso, $labelTela);
                }
            }
        }
    }

    /**
     * @param $aclLabelsJson
     * @param $containerId
     * @param null $a
     * @return mixed|null
     */
    private function findMenuIdByContainerIdInAclLabels($aclLabelsJson, $containerId, &$a = null)
    {
        if (!is_null($a)) {
            return $a;
        }

        foreach ($aclLabelsJson as $item) {
            if ($this->hasLabelAndContainerId($item) && $item['data']['container']['id'] === $containerId) {
                $a = $item['data']['label'];
            }
            if ($this->hasChildren($item)) {
                $this->findMenuIdByContainerIdInAclLabels($item['children'], $containerId, $a);
            }
        }

        return $a;
    }

    /**
     * @param $item
     * @return bool
     */
    private function hasLabelAndContainerId($item): bool
    {
        return isset($item['data']['container']) && isset($item['data']['container']['id']) && isset($item['data']['label']);
    }

    private function createPerfilacessoLabelByOldAcl($label, $nrperfilacesso, $labelTela)
    {
        $filter = array('NRPERFILACESSO' => $nrperfilacesso, 'LABEL' => $label);
        $perfilacessoLabel = $this->perfilacessoLabelRepository->findOneBy($filter);

        if (empty($perfilacessoLabel)) {
            $this->createPerfilacessoLabel($nrperfilacesso, $label, $labelTela);
        }
    }

    /**
     * @param $labels
     * @param $nrperfilacesso
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function savePerfilacessoLabel($labels, $nrperfilacesso, $labelTela)
    {
        $labelsQueryIn = $this->getStringInQueryByArray($labels, 'LABEL');
        $perfilacessoLabels = $this->perfilacessoLabelRepository->findPerfilacessoLabelByNrperfilacessoAndLabels($nrperfilacesso, $labelsQueryIn, getNrorg());

        if (empty($perfilacessoLabels)) {
            foreach ($labels as $label) {
                $this->createPerfilacessoLabel($nrperfilacesso, $label, $labelTela);
            }
        } else {
            foreach ($labels as $label) {
                $key = in_array($label, array_column($perfilacessoLabels, 'LABEL'));
                if ($key === false) {
                    $this->createPerfilacessoLabel($nrperfilacesso, $label, $labelTela);
                }
            }
        }
    }

    /**
     * @param array $labels
     * @param string $column
     * @param string $condition
     * @return string
     */
    private function getStringInQueryByArray(array $labels, string $column, string $condition = 'AND'): string
    {
        $queryIn = "$condition $column IN (";
        foreach ($labels as $label) {
            $queryIn .= "'" . $label . "', ";
        }
        $queryIn = substr($queryIn, 0, -2) . ")";
        return $queryIn;
    }

    /**
     * @param $nrperfilacesso
     * @param $label
     * @param string $idativo
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    private function createPerfilacessoLabel($nrperfilacesso, $label, $labelTela, $idativo = 'S')
    {
        $newCode = InstanceProvider::getNewCode()->getCodeByTableName('PERFILACESSO_LABEL', getNrorg(), 1, 10, true);
        $auditFields = $this->perfilacessoLabelRepository->getAuditFieldValuesDefaultInsert();
        if (empty($labelTela)) {
            $valores = ['Perfil de Acesso', $label, $newCode, $nrperfilacesso];
            $nomes = ["Tela", 'LABEL', 'ID', 'NRPERFILACESSO'];
        } else {
            $valores = [$labelTela['label'], $label, $newCode, $nrperfilacesso];
            $nomes = ["Tela", 'LABEL', 'ID', $labelTela['NRPERFILACESSO']];
        }
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($newCode, $descricao, '1', 'PERFILACESSO_LABEL', '/perfilacessoLabel');   
        $this->perfilacessoLabelRepository->savePerfilacessoLabel(
            $newCode, $label, intval($nrperfilacesso), $auditFields['NRORG'], $idativo, $auditFields['CDOPERINCLUSAO']);
    }

    /**
     * @param $labels
     * @param $nrperfilacesso
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function deletePerfilacessoLabel($labels, $nrperfilacesso, $labelTela)
    {
        $labelsQueryIn = $this->getStringInQueryByArray($labels, 'LABEL');
        $valores = [$labelTela['label'], $nrperfilacesso];
        $nomes = ["Tela", $labelTela['NRPERFILACESSO']];
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($nrperfilacesso, $descricao, '3', 'PERFILACESSO_LABEL', '/perfilacessoLabel');
        $this->perfilacessoLabelRepository->deletePerfilacessoLabel($nrperfilacesso, $labelsQueryIn, getNrorg());
    }
}
